local Layout = FlexBar2:NewModule("Layout");
Layout.Version = 1

Layout.ButtonMixin = FlexBar2.OO.Mixin
{
	"SetPosition",
	"Show",
	"Hide",
    "Lock",
    "Unlock",
    "SetIcon",
    "GetIcon",
};

function Layout.ButtonMixin:Initialize()
	FlexBar2:Debug("               Setting up layout");
	--Actual button creation (note: not setting a NormalTexture here, an apart function does that which is called later on [at the icon creation part])
	local Frame = CreateFrame("CheckButton", "FB2_BTN_" .. self.uid, UIParent, "SecureActionButtonTemplate");
	self.Frame = Frame;
    Frame.Object = self;
	
	-- Strata and sizing, drag registering
	Frame:SetFrameStrata("MEDIUM");
    Frame:SetFrameLevel(1);
	Frame:SetWidth(36);
	Frame:SetHeight(36);

	-- Textures setup
	Frame:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress");
	Frame:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight");
	Frame:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	Frame:GetHighlightTexture():SetBlendMode("ADD");
	Frame:GetCheckedTexture():SetBlendMode("ADD");

	-- Icon setup
	Frame.Icon = self.Frame:CreateTexture(nil, "BACKGROUND")
	Frame.Icon:SetAllPoints(Frame);

	-- Cooldown ticker setup
	Frame.Cooldown = CreateFrame("Cooldown", nil, self.Frame, "CooldownFrameTemplate");
	self.LastCooldown = {};

	-- NormalTexture setup
	self.Frame:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	local NormalTexture = self.Frame:GetNormalTexture();
	NormalTexture:SetWidth(60);
	NormalTexture:SetHeight(60);
	NormalTexture:ClearAllPoints();
	NormalTexture:SetPoint("CENTER", 1, -1);

	-- Button settings
	Frame:RegisterForClicks("AnyUp");
	Frame:EnableMouse(true);
	Frame:SetClampedToScreen(true);
	Frame:SetMovable(false);

    -- Dragger frame
    Frame.Dragger = CreateFrame("Frame");
    local Dragger = Frame.Dragger;
    Dragger.Object = self;
    Dragger:SetFrameStrata("MEDIUM");
    Dragger:SetFrameLevel(3);
    Dragger:SetWidth(23);
    Dragger:SetHeight(23);
    Dragger.texture = Dragger:CreateTexture(nil,'Overlay');
    Dragger.texture:SetTexture(0, 0.6, 0, 0.5);
    Dragger.texture:SetAllPoints(Dragger);
	Dragger:EnableMouse(true);
	Dragger:SetClampedToScreen(true);
	Dragger:SetMovable(true);
    Dragger:SetScript("OnMouseUp", function()
        if(type(self.GroupMemberList) == "table") then
            FlexBar2:Debug("Groupstop");
            local _, _, _, MasterX, MasterY = this:GetPoint();
            for _, GroupMember in pairs(self.GroupMemberList) do
                if(GroupMember ~= self) then
                    FlexBar2:Debug("Stopping dragging for [", _ , "]");
                    local SlaveFrame = GroupMember.Frame.Dragger;
                    local _, _, _, SlaveX, SlaveY = SlaveFrame:GetPoint(); 
                    SlaveFrame.Object:SetPosition(SlaveX + MasterX, -(MasterY + SlaveY));
                    SlaveFrame.Object.InfoTable.X = SlaveX + MasterX;
                    SlaveFrame.Object.InfoTable.Y = -(MasterY + SlaveY);
                end
            end
        end
        this:StopMovingOrSizing();
        local _, _, _, X, Y = this:GetPoint();
        self.InfoTable.X = X;
        self.InfoTable.Y = -Y;
    end);
    Dragger:SetScript("OnMouseDown", function()
        if(type(self.GroupMemberList) == "table") then
            local _, _, _, MasterX, MasterY = this:GetPoint();
            for _, GroupMember in pairs(self.GroupMemberList) do
                if(GroupMember ~= self) then
                    local SlaveFrame = GroupMember.Frame.Dragger;
                    local _, _, _, SlaveX, SlaveY = SlaveFrame:GetPoint();
                    SlaveFrame:SetPoint("TOPLEFT", this, "TOPLEFT", SlaveX - MasterX, SlaveY - MasterY);
                end
            end
        end
        this:StartMoving();
    end);

    Frame:SetAllPoints(Dragger);
end

function Layout.ButtonMixin:Deactivate()
	self:Hide();
end

function Layout.ButtonMixin:SetDefaults()
	self.InfoTable.Show = true;
    self.InfoTable.Lock = true;
    local ButtonNumber = tonumber(self.Name);
    if(ButtonNumber) then
        self.InfoTable.X = 300 + (24 * (ButtonNumber - ((math.ceil(ButtonNumber / 12) - 1) * 12) - 1));
        self.InfoTable.Y = 300 + (24 * (math.ceil(ButtonNumber / 12) - 1));
    else
        self.InfoTable.X = 300;
    	self.InfoTable.Y = 300;
    end
	
end

function Layout.ButtonMixin:LoadSettings()
	self:SetPosition(self.InfoTable.X, self.InfoTable.Y);
	if(self.InfoTable.Show == true) then
		self:Show();
	else
		self:Hide();
	end
end

function Layout.ButtonMixin:SetPosition(X, Y)
	if(type(X) ~= "number") then X = self.InfoTable.X end
	if(type(Y) ~= "number") then Y = self.InfoTable.Y end
    self.Frame.Dragger:SetPoint("TOPLEFT", UIParent, "TOPLEFT", X, -Y);
end

function Layout.ButtonMixin:Show()
	self.Frame:Show();
	self:SetAttribute("showstates", "*", false);
    if(self.InfoTable.Lock == true) then
        self:Lock();
    else
        self:Unlock();
    end
end

function Layout.ButtonMixin:Hide()
	self.Frame:Hide();
    self:Lock();
	self:SetAttribute("showstates", "!*", false);
end

function Layout.ButtonMixin:Lock()
    self.Frame.Dragger:Hide();
end

function Layout.ButtonMixin:Unlock()
    self.Frame.Dragger:Show();
end

function Layout.ButtonMixin:GetIcon(Icon)
    return self.Frame.Icon:GetTexture(Icon);
end

function Layout.ButtonMixin:SetIcon(Icon)
    self.Frame.Icon:SetTexture(Icon);
end

-- GUI --
local Waterfall = FlexBar2.Waterfall;
local Positioning_X, Positioning_Y, Show, Status_Show, Status_Lock, Positioning_AutoAlign;
function Layout.ButtonWaterfall(Action, ButtonName, ButtonTable)
	if(Action == "edit") then
		Status_Show = FlexBar2:GetSetting(ButtonTable, "Show");
        Status_Lock = FlexBar2:GetSetting(ButtonTable, "Lock")
		Waterfall:AddControl(
		"type","checkbox",
		"text","Show button",
		"width",300,
		"causeRefresh", false,
		"getFunc" ,function() return Status_Show; end,
		"setFunc", function(Show) Status_Show = Show; end);
        Waterfall:AddControl(
        "type","checkbox",
		"text","Lock button",
		"width",300,
		"causeRefresh", false,
		"getFunc" ,function() return Status_Lock; end,
		"setFunc", function(Lock) Status_Lock = Lock; end);
		Waterfall:AddControl(
		"type", "label",
		"text", "Button Position");
		Waterfall:AddControl(
		"type", "label",
		"text", "X: ");
		Waterfall:AddControl(
		"type", "textbox",
		"width", "50",
		"noNewLine", true,
		"causeRefresh", false,		
		"changedFunc", function(X) Positioning_X = X; end,
		"getFunc", function() if(#ButtonTable >= 2) then return "" else return FlexBar2:GetSetting(ButtonTable, "X") or 300; end end);
		Waterfall:AddControl(
		"type", "label",
		"text", "Y: ");
		Waterfall:AddControl(
		"type", "textbox",
		"width", "50",
		"noNewLine", true,
		"causeRefresh", false,		
		"changedFunc", function(Y) Positioning_Y = Y; end,
		"getFunc",  function() if(#ButtonTable >= 2) then return "" else return FlexBar2:GetSetting(ButtonTable, "Y") or 300; end end);
		if(#ButtonTable >= 2) then
			Waterfall:AddControl(
			"type","checkbox",
			"text","AutoAlign",
			"width",300,
			"causeRefresh", false,
			"getFunc" ,function() return Positioning_AutoAlign; end,
			"setFunc", function(AutoAlign) Positioning_AutoAlign = AutoAlign; end);
		end
	elseif(Action == "save") then
        local ButtonList = FlexBar2.Buttons;
		if(Positioning_X ~= "" and Positioning_Y ~= "") then
			if(Positioning_AutoAlign == true) then
                local X = Positioning_X + (24 * (ButtonName - ((math.ceil(ButtonName / 12) - 1) * 12) - 1));
                local Y = Positioning_Y + (24 * (math.ceil(ButtonName / 12) - 1));
                FlexBar2:ApplySetting(ButtonTable, "X", X);
                FlexBar2:ApplySetting(ButtonTable, "Y", Y);
                ButtonList[ButtonTable]:SetPosition(X, Y);
			else
				FlexBar2:ApplySetting(ButtonTable, "X", Positioning_X);
				FlexBar2:ApplySetting(ButtonTable, "Y", Positioning_Y);
                ButtonList[ButtonTable]:SetPosition(Positioning_X, Positioning_Y);
			end 
		end
        FlexBar2:ApplySetting(ButtonTable, "Show", Status_Show);
        if(Status_Show == true) then
            ButtonList[ButtonTable]:Show();
        else
            ButtonList[ButtonTable]:Hide();
        end
        FlexBar2:ApplySetting(ButtonTable, "Lock", Status_Lock);
        if(Status_Lock == true) then
            ButtonList[ButtonTable]:Lock();
        else
            ButtonList[ButtonTable]:Unlock();
        end
	end
end

-- SlashCommands --
local SlashCommands = FlexBar2.SlashCommands;
local function SlashCommand_Show(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Show", true);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Show();
	end
end

local function SlashCommand_Hide(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Show", false);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Hide();
	end
end


local function SlashCommand_Lock(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Lock", true);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Lock();
	end
end

local function SlashCommand_Unlock(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	FlexBar2:ApplySetting(ButtonTable, "Lock", false);
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:Unlock();
	end
end


local function SlashCommand_SetPosition(Params)
	FlexBar2:Debug(ButtonList);
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	if(Params.smart == "false") then
		FlexBar2:ApplySetting(ButtonTable, "X", Params.x);
		FlexBar2:ApplySetting(ButtonTable, "Y", Params.y);
	elseif(Params.smart == "true") then
		for Number, Button in ipairs(ButtonTable) do
			FlexBar2:ApplySetting(Button, "X", Params.x + (24 * (Number - ((math.ceil(Number / 12) - 1) * 12) - 1)));
			FlexBar2:ApplySetting(Button, "Y", Params.y + (24 * (math.ceil(Number / 12) - 1)));
		end
	end
	local ButtonList = FlexBar2.Buttons;
	for _, ButtonName in ipairs(ButtonTable) do
		ButtonList[ButtonName]:SetPosition();
	end
end

local VerifyExistance = FlexBar2.SlashCommands.VerifyExistance;
SlashCommands:AddCommand("show", SlashCommand_Show, "layout", "/FlexBar2 show button=<Button>");
	SlashCommands:AddParam("show", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("hide", SlashCommand_Hide, "layout", "/FlexBar2 hide button=<Button>");
	SlashCommands:AddParam("hide", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("lock", SlashCommand_Lock, "layout", "/FlexBar2 lock button=<Button>");
	SlashCommands:AddParam("lock", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("unlock", SlashCommand_Unlock, "layout", "/FlexBar2 unlock button=<Button>");
	SlashCommands:AddParam("unlock", "button", true, nil, nil, VerifyExistance)
SlashCommands:AddCommand("setposition", SlashCommand_SetPosition, "layout", "/FlexBar2 setposition button=<Button> x=<x coordinate> y=<y coordinate> (smart=<true/false>)");
	SlashCommands:AddParam("setposition", "button", true, nil, nil, VerifyExistance)
	SlashCommands:AddParam("setposition", "x", false, nil, "300", nil);
	SlashCommands:AddParam("setposition", "y", false, nil, "300", nil);
	SlashCommands:AddParam("setposition", "smart", false, {"true", "false"}, "true", nil);
